-- root game entity
Master = Class:extend()

function Master:new(image,x,y,x2,y2)
	self.image = love.graphics.newImage(image)
	self.quad = 16
	
	self.w = self.image:getWidth()
	self.h = self.image:getHeight()
	
	self.x = x or x2*self.quad
	self.y = y or y2*self.quad
	self.x2 = x2 or math.floor((x/self.quad) + 0.5)
	self.y2 = y2 or math.floor((y/self.quad) + 0.5)
	
	--[[ default animation values
	self.f = {}
	self.fTotal = 1
	self.fSpeed = 3
	self.fCurrent = 1
	]]
	-- default variant values
	self.var = {}
	self.varTotal = 1
	self.varCurrent = 1
	
	-- velocity variables
	--self.vx = 0
	--self.vy = 0
	
	-- mass/weight variable
	--self.mass = 0
end

function Master:update(dt)
	--[[ animation frame check and upticking
	if self.fTotal >= 2 then
		self.fCurrent = self.fCurrent + self.fSpeed * dt
		if self.fCurrent >= self.fTotal + 1 then
			self.fCurrent = 1
		end
	end
	]]
	-- movement code
	--self.x = self.x + self.vx * dt
	--self.y = self.y + self.vy * dt
end

function Master:draw()
	-- basic block drawing, checks variant but not connections
	love.graphics.draw(self.image, self.var[self.varCurrent], self.x, self.y)
	
	
	--[[ has animation
	if self.fTotal >= 2 then
		love.graphics.draw(self.image, self.f[math.floor(self.fCurrent)], self.x, self.y)
	end
	]]
end

-- check and fill image variants
function Master:fillVariants()
	for i = 0, self.varTotal - 1 do
		local v = (self.quad+1)*i
			
		-- fill self.var
		table.insert(self.var,love.graphics.newQuad(1,1+v,self.quad,self.quad,self.w,self.h))
	end
end

--[[ check and fill animation frames
function Master:fillFrames()
	if self.fTotal >= 2 then
		-- puts animated frames below variants in the spritesheet
		local varSpace = (self.varTotal*self.quad + self.varTotal)*self.varTotal
		
		for i = 0, self.fTotal - 1 do
			table.insert(self.f, love.graphics.newQuad(1, 1 + varSpace + i * (self.quad + 1), self.quad, self.quad, self.w, self.h))
		end
	end
end
]]
-- block template
Block = Master:extend()

function Block:new(image,x,y,x2,y2,var)
	Block.super.new(self,image,x,y,x2,y2)
	
	self.varTotal = 4
	self.varCurrent = var or math.floor(math.random(1, self.varTotal))
	self:fillVariants()
	
	-- none0, top1, right2, topRight3, bottom4, vertical5, bottomRight6, fullRight7, left8, topLeft9, sides10, fullTop11, bottomLeft12, fullLeft13, fullBottom14, full15
	self.merge = 0
	
	--self.rotation = 1
end

function Block:update(t,state,dt)
	-- check adjacencies and update self.merge
	-- v (self)
	for i = 1, #t do
		local v = t[i]
		local v2 = t[i+1]
		local v3 = t[i+state.worldW]
		
		-- check if not edge of the world
		if i % state.worldW ~= 0
		-- check if v2 fits in the table
		and i + 1 <= #t
		-- compare v and v2
		and v.id == v2.id then
			if v.merge == 0
			or v.merge == 1
			or v.merge == 4
			or v.merge == 5
			or v.merge == 8
			or v.merge == 9
			or v.merge == 12
			or v.merge == 13 then
				v.merge = v.merge + 2
			end
			if v2.merge == 1 then
				v2.merge = v2.merge + 8
			else
				v2.merge = 8
			end
		end
		
		-- check if v3 fits in the table
		if i + state.worldW <= #t then
		-- compare v and v3               -- just this needs fixing for block connecting to be done
		--and v.id == v3.id then
			-- check if v is unmerged
			if v.merge == 0
			or v.merge == 1
			or v.merge == 2
			or v.merge == 3
			or v.merge == 8
			or v.merge == 9
			or v.merge == 10
			or v.merge == 11 then
				v.merge = v.merge + 4
			end
			v3.merge = 1
		end
	end
end

function Block:draw()
	-- same as in Master:draw(), but takes into account connections
	love.graphics.draw(self.image, self.var[self.varCurrent][self.merge+1], self.x, self.y)

	--[[ has animation, same as Master:draw()
	if self.fTotal >= 2 then
		love.graphics.draw(self.image, self.f[math.floor(self.fCurrent)], self.x, self.y)
	end
	]]
end

-- check and fill image variants
function Block:fillVariants()
	-- same as Master:fillVariants(), but takes into account block connecting
	for i = 0, self.varTotal - 1 do
		local v = (self.quad+1)*i
		
		local mergeVar = {}
		
		for j = 0, 15 do
			local v2 = (self.quad+1)*j
			
			-- fill mergeVar with all tile connections
			table.insert(mergeVar,love.graphics.newQuad(1+v2,1+v,self.quad,self.quad,self.w,self.h))
		end
		
		-- add mergeVar to variants table
		table.insert(self.var,mergeVar)
	end
end