-- root ui entity
UI = Class:extend()

function UI:new(image, x, y)
	self.image = love.graphics.newImage(image)
	self.wQuad = 8
	self.hQuad = 8
	
	self.w = self.image:getWidth()
	self.h = self.image:getHeight()
	self.wO = self.w / 2
	self.hO = self.h / 2
	
	self.x = x
	self.y = y
--	self.xO = self.x + self.wO
--	self.yO = self.y + self.hO
	self.xS = 1
	self.yS = 1
	
	-- default animation frame values
	self.f = {}
	self.fTotal = 1
	self.fSpeed = 3
	self.fCurrent = 1
end

function UI:update(dt)
	-- animation frame check and upticking
	if self.fTotal >= 2 then
		self.fCurrent = self.fCurrent + self.fSpeed * dt
		if self.fCurrent >= self.fTotal + 1 then
			self.fCurrent = 1
		end
	end
end

function UI:draw()
	-- drawing type
	if self.fTotal >= 2 then
		love.graphics.draw(self.image, self.f[math.floor(self.fCurrent)], self.x, self.y, 0, self.xS, self.yS, self.wO, self.hO)
	else
		love.graphics.draw(self.image, self.x, self.y, 0, self.xS, self.yS, self.wO, self.hO)
	end
end

-- check and fill animation frames
function UI:fillFrames()
	if self.fTotal >= 2 then
		for i = 0, self.fTotal - 1 do
			table.insert(self.f, love.graphics.newQuad(1 + i * (self.wQuad + 1), 1, self.wQuad, self.hQuad, self.w, self.h))
		end
	end
end

-- button template
Button = UI:extend()

function Button:new(image, x, y)
	Button.super.new(self, image, x, y)
	
	self.isHovered = false
	self.isClicked = false
	self.isReleased = true
end

function Button:update(image, x, y)
	Button.super.update(self, dt)
	clickCheck(self)
	
	-- scale button if hovered over or clicked
	if self.isHovered then
		if self.isClicked then
			self.xS, self.yS = 0.9, 0.9
		else
			self.xS, self.yS = 1, 1
		end
	else
		self.xS, self.yS = 0.9, 0.9
	end
end

-- useful UI functions
function hoverCheck(a)
	mouseX,mouseY = love.mouse.getPosition()
	
	-- check if mouse is over button
	if mouseX > a.x - a.wO
	and mouseY > a.y - a.hO
	and mouseX < a.x + a.wO
	and mouseY < a.y + a.hO then
		a.isHovered = true
		return true
	else
		a.isHovered = false
		return false
	end
end

function clickCheck(a)
	if hoverCheck(a) then
		if love.mouse.isDown(1) then
			a.isClicked = true
			a.isReleased = false
			return true
		else
			a.isClicked = false
			a.isReleased = true
			return false
		end
	else
		if a.isClicked then
			a.isClicked = false
			a.isReleased = true
		end
	end
end