Camera = require 'camera'
Class = require 'classic'
Gamestate = require 'gamestate'
Timer = require 'timer'
--Vector = require 'vector'

require 'Entities.master'
require 'Entities.Blocks.blocks'
require 'UI.ui'

-- gamestates
local menu = {}
local game = {}
local pause = {}
local settings = {}

-- menu code
function menu:init()
	-- menu init
	self.buttons = {}
	self.buttonImgs = {
		'UI/start.png',
		'UI/options.png',
		'UI/quit.png'
	}
	self.w, self.h = love.window.getDesktopDimensions()
	self.wO, self.hO = self.w / 2, self.h / 2
	
	-- fill menu with buttons
	local lastButton
	for i = 1, #self.buttonImgs do
		local v = self.buttonImgs[i]
		if lastButton == nil then
			table.insert(self.buttons, Button(v, self.wO, self.hO))
		else
			table.insert(self.buttons, Button(v, self.wO, lastButton.y + lastButton.h + lastButton.hO))
		end
		lastButton = self.buttons[i]
	end
end

function menu:enter(from)
--	self.from = from
	Timer.after(0.1, function() self.buttons.active = true end)
end

function menu:resume(from)
--	self.from = from
	Timer.after(0.1, function() self.buttons.active = true end)
end

function menu:leave()
	self.buttons.active = false
end

-- game code
function game:init()
	-- game init
	self.cam = Camera()
	self.ents = {}
	
	-- set worldSize
	self.worldW = 32
	self.worldH = 4
	
	-- temporary
	generateTerrain()
end

function game:leave()
	
end

-- pause menu code
function pause:init()
	self.buttons = {}
	self.buttonImgs = {
		'UI/resume.png',
		'UI/options.png',
		'UI/mainMenu.png'
	}
	self.w, self.h = love.window.getDesktopDimensions()
	self.wO,self.hO = self.w / 2, self.h / 2
	
	-- fill menu with buttons
	local lastButton
	for i = 1, #self.buttonImgs do
		local v = self.buttonImgs[i]
		if lastButton == nil then
			table.insert(self.buttons, Button(v, self.wO, self.hO))
		else
			table.insert(self.buttons, Button(v, self.wO, lastButton.y + lastButton.h + lastButton.hO))
		end
		lastButton = self.buttons[i]
	end
end

function pause:enter(from)
--	self.from = from
	Timer.after(0.1, function() self.buttons.active = true end)
end

function pause:resume(from)
--	self.from = from
	Timer.after(0.1, function() self.buttons.active = true end)
end

function pause:leave()
	self.buttons.active = false
end

-- pause menu code
function settings:init()
	self.buttons = {}
	self.buttonImgs = {
		'UI/options.png',
		'UI/options.png',
		'UI/back.png'
	}
	self.w, self.h = love.window.getDesktopDimensions()
	self.wO,self.hO = self.w / 2, self.h / 2
	
	-- fill menu with buttons
	local lastButton
	for i = 1, #self.buttonImgs do
		local v = self.buttonImgs[i]
		if lastButton == nil then
			table.insert(self.buttons, Button(v, self.wO, self.hO))
		else
			table.insert(self.buttons, Button(v, self.wO, lastButton.y + lastButton.h + lastButton.hO))
		end
		lastButton = self.buttons[i]
	end
end

function settings:enter(from)
--	self.from = from
	Timer.after(0.1, function() self.buttons.active = true end)
end

function settings:resume(from)
--	self.from = from
	Timer.after(0.1, function() self.buttons.active = true end)
end

function settings:leave()
	self.buttons.active = false
end

-- engine code
function love.load()
	Gamestate.registerEvents()
	Gamestate.switch(menu)
end

function love.update(dt)
	-- gamestate updating
	local state = Gamestate.current()
	-- timer updating
	Timer.update(dt)
	
	-- main menu
	if state == menu
	and state.buttons.active then
		updateTable(state.buttons, dt)
		
		-- menu buttons
		if state.buttons[1].isClicked then
			-- start game
			return Gamestate.switch(game)
		elseif state.buttons[2].isClicked then
			-- open settings
			state.buttons.active = false
			return Gamestate.push(settings)
		elseif state.buttons[3].isClicked then
			-- quit game
			love.event.quit()
		end
	
	-- game
	elseif state == game then
		updateTable(state.ents[1],state,dt)
		camControls(state, 100, dt)
	
	-- pause menu
	elseif state == pause
	and state.buttons.active then
		updateTable(state.buttons, dt)
		
		-- pause menu buttons
		if state.buttons[1].isClicked then
			-- resume game
			return Gamestate.pop(state)
		elseif state.buttons[2].isClicked then
			-- open settings
			state.buttons.active = false
			return Gamestate.push(settings)
		elseif state.buttons[3].isClicked then
			-- quit to menu
			return Gamestate.switch(menu)
		end
		
	-- settings menu
	elseif state == settings
	and state.buttons.active then
		updateTable(state.buttons, dt)
		
		-- settings menu buttons
		if state.buttons[1].isClicked then
			
		elseif state.buttons[2].isClicked then
			
		elseif state.buttons[3].isClicked then
			-- return to previous screen
			return Gamestate.pop(state)
		end
	end
end

function love.draw()
	-- gamestate drawing
	local state = Gamestate.current()
	
	-- main menu
	if state == menu
	and state.buttons.active then
		drawTable(state.buttons)
	
	-- game
	elseif state == game then
		-- draw entities through cam
		state.cam:attach()
			drawTable(state.ents[1])	
		state.cam:detach()
		
		-- TESTING
		-- top layer
		for i = 0, game.worldW - 1 do
			local j = i + 1
			love.graphics.print(game.ents[1][j].merge,i*20,0)
		end
		
		-- layer below
		for i = 0, game.worldW - 1 do
			local j = i + 1 + game.worldW
			love.graphics.print(game.ents[1][j].merge,i*20,20)
		end
		
		-- bottom layer
		for i = 0, game.worldW - 1 do
			local j = i + 1 + game.worldW*2
			love.graphics.print(game.ents[1][j].merge,i*20,60)
		end
	
	elseif state == pause
	and state.buttons.active then
		drawTable(state.buttons)
	
	elseif state == settings
	and state.buttons.active then
		drawTable(state.buttons)
	end
end

function love.keyreleased(key)
	-- gamestate switching
	local state = Gamestate.current()
	
	--menu
	if state == menu
	and state.buttons.active then
		updateTable(state.buttons, dt)
		-- menu buttons
		if key == 'escape' then
			-- quit game
			love.event.quit()
		end
		
	-- game
	elseif state == game then
		if key == 'escape' then
			-- pause
			return Gamestate.push(pause)
		end
		
	-- pause menu
	elseif state == pause
	and key == 'escape' then
		-- return to previous screen
		return Gamestate.pop(state)
	
	-- settings menus
	elseif state == settings
	and key == 'escape' then
		-- return to previous screen
		return Gamestate.pop(state)
	end
end

-- useful -- functions --

-- update all objects in a table
function updateTable(t,state,dt)
	for i = 1, #t do
		local v = t[i]
		v:update(t,state,dt)
	end
end

-- draw all objects in a table
function drawTable(t)
	for i = 1, #t do
		local v = t[i]
		v:draw()
	end
end

-- cam controls
function camControls(a, speed, dt)
	-- xy movement
	if love.keyboard.isDown('w') then
		-- move cam left
		a.cam:move(0, -speed * dt)
	end
	if love.keyboard.isDown('a') then
		-- move cam left
		a.cam:move(-speed * dt, 0)
	end
	if love.keyboard.isDown('d') then
		-- move cam right
		a.cam:move(speed * dt, 0)
	end
	if love.keyboard.isDown('s') then
		-- move cam left
		a.cam:move(0, speed * dt)
	end
end

-- OOD -- TESTING -- CODE --
-- terrain gen (all kinds of fucked, just for testing)
function generateTerrain()
	local blocks = {}
	local yOffset = 32
	
	for i = 0, game.worldH - 1 do
		-- generate Obsidian
		for j = 0, game.worldW - 1 do
			generateBlock(blocks,Obsidian,j,i+yOffset)
		end
		-- generate Peridotite
		for j = 0, game.worldW -1 do
			generateBlock(blocks,Peridotite,j,i+yOffset+game.worldH)
		end
	end
	
	--initBlocks(blocks)
	
	-- swap blocks out while keeping all the previous block's values
	--blocks[1] = Air(blocks[1].x2,blocks[1].y2,blocks[1].varCurrent,blocks[1].merge)
	
	--[[
	for i = 1, #blocks do
		local rand = math.floor(love.math.random()+0.5)
		if rand == 1 then
			blocks[i] = Air(blocks[i].x2,blocks[i].y2,blocks[i].varCurrent)
		end
		
		--blocks[i].merge = 15
	end
	]]
	-- move blocks to game.ents
	table.insert(game.ents,blocks)
end

function generateBlock(t,block,blockX,blockY)
	table.insert(t,block(blockX,blockY))
end